from datetime import datetime, timezone
import requests
import json

class TimeRange:
    #TODO: need to handle defualt paramaters and the call to timestamp()
    def __init__(self, end: datetime = 0, start: datetime = 0):
        self.end = end
        self.start = start

    def to_json(self):
        return dict(end=int(self.end.timestamp()), start=int(self.start.timestamp()))


class PropertyFilter:

    def __init__(self, file_name: str, host_name: str):
        self.file_name = file_name
        self.host_name = host_name

    def to_json(self):
        return dict(fileName=self.file_name, hostName=self.host_name)


class Scope:
    """
    Name            Type            Required    Description
    filter	        PropertyFilter	Yes		    The optional filters with which to limit the query.
    hostGroupID	    Number	        No		    Get the host group ID that this query should be scoped by.
    smartFolderID	Number	        No		    Get the smart folder ID that this query should be scoped by.
    timeRange	    TimeRange	    Yes		    The time range with which to limit the query.

    """
    def __init__(self, filter:PropertyFilter, time_range: TimeRange, host_group_id = 0, smart_folder_id = None):
        self.filter = filter
        self.time_range = time_range
        self.host_group_id = host_group_id
        self.smart_folder_id = smart_folder_id

    def to_json(self):
        return dict(filter=self.filter.to_json(), timeRange=self.time_range.to_json())


class LiftApplicationDriftRequest:

    def __init__(self, scope):
        self.scope = scope

    def to_json(self):
        return json.dumps(dict(ListApplicationDriftRequest=dict(scope=self.scope.to_json())))


class AddGlobalRulesetRulesRequest:

    def __init__(self, hash, description=""):
        self.hash = hash
        self.description = description

    def to_json(self):
        return json.dumps(dict(AddGlobalRulesetRulesRequest=dict(rules=[dict(sha256=self.hash, action="block", description=self.description)])))





import requests
from ..models.host import Host


class HostUtils:


    def __init__(self, config):
        self.config = config
        self.api_url = self.config.rest_url() + "hosts"
        self.headers = {'Accept': 'application/json'}


    def create_host(self, hostTransport) -> Host:
        host = Host(hostTransport.ID, hostTransport.name, hostTransport.description, hostTransport.displayName, hostTransport.external, \
                    hostTransport.externalID, hostTransport.hostGroupID, hostTransport.hostType, hostTransport.platform, \
                    hostTransport.securityProfileID)
        return host


    def components(self, host_id, sessionId, verify_ssl:bool = False):
        url = "{}/{}/components".format(self.api_url, host_id)
        response = requests.get(url, verify=verify_ssl, headers=self.headers, params={'sID': sessionId})
        return response.json()



from suds import Client


class IDFilter():

    def __init__(self, id, operator, client):
        self.id = id
        self.operator = operator
        self.client = client

    def get_transport(self):
        idft = self.client.factory.create('IDFilterTransport')
        idft.id = self.id

        eo = self.client.factory.create('EnumOperator')
        operators = {"GREATER_THAN": eo.GREATER_THAN, "LESS_THAN": eo.LESS_THAN, "EQUAL":eo.EQUAL }

        if operators[self.operator]:
            idft.operator = operators[self.operator]
        else:
            idft.operator = eo.GREATER_THAN


        return idft



from datetime import datetime, timezone
import json
import time

import requests

class ModifyTrustedUpdateModeRequest:

    def __init__(self, duration: int, enabled: bool):
        self.duration = duration
        self.enabled = enabled

    def to_json(self):
        return json.dumps(dict(ModifyTrustedUpdateModeRequest=dict(duration=self.duration, enabled=self.enabled)))







class Config:

    def __init__(self, host, port, soap_api_wsdl='webservice/Manager?WSDL'):
        self.host = host
        self.port = port
        self.soap_api_wsdl = soap_api_wsdl

    def soap_url(self):
        return "https://{}:{}/{}".format(self.host, self.port, self.soap_api_wsdl)

    def rest_url(self):
        return "https://{}:{}/rest/".format(self.host, self.port)




import sys
import datetime
import urllib3
import json
import requests

class SmartCheck():

    def __init__(self, username: str, password: str, host: str, port: int = '443',
                 verify_ssl:bool = False, cacert_file:str = False):
        """

               :param username:
               :param password:
               :param host:
               :param port:
               :param verify_ssl:
               :param cacert_file: optional CA certificates to trust for certificate verification
               """

        self.headers = {'Content-Type': 'application/json'}
        self._username = username
        self._password = password
        self.host = host
        self.port = port
        self.verify_ssl = verify_ssl
        self.cacer_file = cacert_file
        self.token = None
        self.token_expires = None
        self.password_change_require = False
        urllib3.disable_warnings()


        try:
            self.session_id = self.__authenticate()
        except Exception as ex:
            print("Authentication error: ", ex)
            sys.exit()

    def __authenticate(self) -> str:
        credentials = json.dumps(dict(user=dict(userID=self._username, password=self._password)))
        url = "https://{}:{}/api/sessions".format(self.host, self.port)
        r = requests.post(url, data=credentials, verify=self.verify_ssl, headers=self.headers)
        r_json = json.loads(r.content.decode('utf-8'))
        self.token = r_json['token'] if r_json else None
        self.token_expires = datetime.datetime.strptime(r_json['expires'], '%Y-%m-%dT%H:%M:%SZ') if self.token else None
        self.password_change_require = r_json['user']['passwordChangeRequired']
        self.headers['Authorization'] = "Bearer %s" % self.token
        return r.content.decode('utf-8')


    def get_sessions(self):
        """
        Retrieve a list of sessions.

        :return: json object with sessions
        """
        url = "https://{}:{}/api/sessions".format(self.host, self.port)
        self.headers['Authorization'] = "Bearer %s" % self.token
        r = requests.get(url, verify=self.verify_ssl, headers=self.headers)

        return json.loads(r.content.decode('utf-8'))


    def get_users(self):
        """
        Retrieve a list of users.

        :return: json object with users
        """
        url = "https://{}:{}/api/users".format(self.host, self.port)
        self.headers['Authorization'] = "Bearer %s" % self.token
        r = requests.get(url, verify=self.verify_ssl, headers=self.headers)
        return json.loads(r.content.decode('utf-8'))

    def get_scans(self, id=None, registry=None, repository=None, tag=None, exact=False):
        """
        Retrieve a list of scans.

        repository: (Optional) When present, the tag query parameter will filter the list of scans to those scans where the source.repository
                    contains the provided value. If the exact query parameter is also provided (and true), the filter will do
                    an exact match on the value.
        exact: (Optional) When present (and true), any filtering done using the registry, repository, and tag query parameters will
               be done using exact matches.

        :return: json object with scans
        """
        if id == None:
            url = "https://{}:{}/api/scans".format(self.host, self.port)
        else:
            url = "https://{}:{}/api/scans/{}".format(self.host, self.port, id)

        self.headers['Authorization'] = "Bearer %s" % self.token
        if repository:
            self.headers['registry'] = registry
            self.headers['repository'] = repository
            self.headers['tag'] = tag
            self.headers['exact'] = str(exact).lower()

        r = requests.get(url, verify=self.verify_ssl, headers=self.headers)
        return json.loads(r.content.decode('utf-8'))

    def get_scan_malware_findings(self, scan_id, layer_id):
        url = "https://{}:{}/api/scans/{}/layers/{}/malware".format(self.host, self.port, scan_id, layer_id)
        self.headers['Authorization'] = "Bearer %s" % self.token
        params = dict(id=scan_id, layerID=layer_id)
        r = requests.get(url, params=params, verify=self.verify_ssl, headers=self.headers)
        return json.loads(r.content.decode('utf-8'))

    def initiate_scan(self, registry, repository, tag, username=None, password=None, token=None, type="docker"):
        url = "https://{}:{}/api/scans".format(self.host, self.port)
        credentails = dict(username=username, password=password, token=token)
        self.headers['Authorization'] = "Bearer %s" % self.token
        request = dict(source=dict(type=type, registry=registry, repository=repository, tag=tag, credentials=credentails, insecureSkipVerify=self.verify_ssl))
        r = requests.post(url, data=json.dumps(request), verify=self.verify_ssl, headers=self.headers)
        return json.loads(r.content.decode('utf-8'))

    def initiate_scan_ecr(self, registry, repository, tag, region, access_key, secret_access_key, token=None, type="docker"):
        url = "https://{}:{}/api/scans".format(self.host, self.port)
        credentials = dict(aws=dict(region=region, accessKeyId=access_key, secretAccessKey=secret_access_key), token=token)
        self.headers['Authorization'] = "Bearer %s" % self.token
        request = dict(source=dict(type=type, registry=registry, repository=repository, tag=tag, credentials=credentials,insecureSkipVerify=self.verify_ssl))
        r = requests.post(url, data=json.dumps(request), verify=self.verify_ssl, headers=self.headers)
        return json.loads(r.content.decode('utf-8'))

    def get_registries(self):
        url = "https://{}:{}/api/registries".format(self.host, self.port)
        r = requests.get(url, verify=self.verify_ssl, headers=self.headers)
        return json.loads(r.content.decode('utf-8'))

    def get_vulnerabilities(self, id, layer_id):
        """
        Get scan vulnerability findings.

        :param id: scan id
        :param layer_id: scan layer id
        :return: python dict containing vulnerability findings.

        """
        url = "https://{}:{}/api/scans/{}/layers/{}/vulnerabilities".format(self.host, self.port, id, layer_id)
        r = requests.get(url, verify=self.verify_ssl, headers=self.headers)

        return json.loads(r.content.decode('utf-8'))





class IPList:
    """Represents a Deep Security IPList"""
    def __init__(self, id, name, description, ips):
        self.id = id
        self.name = name
        self.description = description
        self.ips = ips



from datetime import datetime
import requests
import json

class TimeRange:
    #TODO: need to handle defualt paramaters and the call to timestamp()
    def __init__(self, end: datetime = 0, start: datetime = 0):
        self.end = end
        self.start = start

    def to_json(self):
        return dict(end=self.end.timestamp(), start=self.start.timestamp())


class PropertyFilter:

    def __init__(self, file_name: str, host_name: str):
        self.file_name = file_name
        self.host_name = host_name

    def to_json(self):
        return dict(fileName=self.file_name, hostName=self.host_name)


class Scope:
    def __init__(self, filter:PropertyFilter, time_range: TimeRange, host_group_id = 0, smart_folder_id = None):
        self.filter = filter
        self.time_range = time_range
        self.host_group_id = host_group_id
        self.smart_folder_id = smart_folder_id

    def to_json(self):
        return dict(filter=self.filter.to_json(), timeRange=self.time_range.to_json())


class ReviewApplicationDriftRequest:

    def __init__(self, scope: Scope, action: str ='allow'):
        self.scope = scope
        self.action = action

    def to_json(self):
        return json.dumps(dict(ReviewApplicationDriftRequest=dict(scope=self.scope.to_json(), action=self.action)))


class DescribeApplicationRequest:

    def __init__(self, scope):
        self.scope = scope

    def to_json(self):
        return json.dumps(dict(DescribeApplicationRequest=dict(scope=self.scope.to_json())))






import collections
import re
from typing import List

from ..models.portlist import PortList


def parse_port_lists(portlists) -> List[PortList]:
    """
    Used to turn a list if PortListTransport objects received by DSM into PortList objects

    :param port_lists: PortListTransport
    :return: List[PortList]
    """
    port_lists = []

    for port_list in portlists:
        items = {}

        if isinstance(port_list.items, collections.Iterable):
            splits = port_list.items.replace(" ", "").strip().split("\n")
            splits = [x for x in splits if re.search(r'\d+', x)]   #removes comment only entries


            if len(splits) == 1 and "," in splits[0]:
                result = splits[0].split(",")
                splits = []
                for i in result:
                    splits.append(i)


            for a in splits:
                splits2 = a.split("#")

                if len(splits2) == 1:
                    items[splits2[0]] = ""
                else:
                    items[splits2[0]] = splits2[1]

        else:
            items = {}


        pl = PortList(port_list.ID, port_list.name, port_list.description, items, port_list.TBUID)
        port_lists.append(pl)


    return port_lists





class Host:
    """Represents a Deep Security Host"""
    def __init__(self, ID, name, description, displayName, external, externalID, hostGroupID, hostType, platform, securityProfileID):
        self.id = ID
        self.name = name
        self.description = description
        self.displayName = displayName
        self.external = external
        self.externalID = externalID
        self.hostGroupID = hostGroupID
        self.hostType = hostType
        self.platform = platform
        self.securityProfileId = securityProfileID


class HostFilter:
    def __init__(self, client, hostGroupId=None, host_id=None, securityProfileId=None, type=None):
        self.hostGroupID = hostGroupId
        self.hostID = host_id
        self.securityProfileID = securityProfileId
        self.type = type  #EnumHostFilterType
        self.client = client

    def get_transport(self):
        hft = self.client.factory.create('HostFilterTransport')
        hft.hostGroupID = self.hostGroupID
        hft.hostID = self.hostID
        hft.securityProfileID = self.securityProfileID

        ehft = self.client.factory.create('EnumHostFilterType')
        types = {"ALL_HOSTS": ehft.ALL_HOSTS,
                 "HOSTS_IN_GROUP": ehft.HOSTS_IN_GROUP,
                 "HOSTS_USING_SECURITY_PROFILE": ehft.HOSTS_USING_SECURITY_PROFILE,
                 "HOSTS_IN_GROUP_AND_ALL_SUBGROUPS": ehft.HOSTS_IN_GROUP_AND_ALL_SUBGROUPS,
                 "SPECIFIC_HOST": ehft.SPECIFIC_HOST,
                 "MY_HOSTS": ehft.MY_HOSTS}

        if self.type is not None and types[self.type]:
            hft.type = types[self.type]
        elif self.hostID is not None:
            hft.type = ehft.SPECIFIC_HOST
        elif self.hostGroupID is not None:
            hft.type = ehft.HOSTS_IN_GROUP_AND_ALL_SUBGROUPS
        elif self.securityProfileID is not None:
            hft.type = ehft.HOSTS_USING_SECURITY_PROFILE
        else:
            hft.type = ehft.ALL_HOSTS

        return hft



"""Helper functions and classes related to SSL contexts.
**Note**: This module is mostly intended for the internal use in
python-icat.  Most users will not need to use it directly or even care
about it.
"""

import ssl
from urllib.request import HTTPSHandler
import suds.transport.http
import urllib3


def create_ssl_context(verify=True, cafile=None, capath=None):
    """Set up the SSL context.
    """
    # This is somewhat tricky to do it right and still keep it
    # compatible across various Python versions.

    try:
        # The easiest and most secure way.
        # Requires either Python 2.7.9 or 3.4 or newer.
        context = ssl.create_default_context(cafile=cafile, capath=capath)
        if not verify:
            context.check_hostname = False
            context.verify_mode = ssl.CERT_NONE
    except AttributeError:
        # ssl.create_default_context() is not available.
        try:
            context = ssl.SSLContext(ssl.PROTOCOL_TLSv1_2)
        except AttributeError:
            # We don't even have the SSLContext class.  This smells
            # Python 2.7.8 or 3.1 or older.  Bad luck.
            return None

        context.options |= ssl.OP_NO_SSLv2
        context.options |= ssl.OP_NO_SSLv3
        if verify:
            context.verify_mode = ssl.CERT_REQUIRED
            if cafile or capath:
                context.load_verify_locations(cafile, capath)
            else:
                context.set_default_verify_paths()
        else:
            context.verify_mode = ssl.CERT_NONE
    return context


class HTTPSTransport(suds.transport.http.HttpTransport):
    """A modified HttpTransport using an explicit SSL context.
    """

    def __init__(self, context, **kwargs):
        """Initialize the HTTPSTransport instance.
        :param context: The SSL context to use.
        :type context: :class:`ssl.SSLContext`
        :param kwargs: keyword arguments.
        :see: :class:`suds.transport.http.HttpTransport` for the
            keyword arguments.
        """
        suds.transport.http.HttpTransport.__init__(self, **kwargs)
        self.ssl_context = context
        self.verify = (context and context.verify_mode != ssl.CERT_NONE)

    def u2handlers(self):
        """Get a collection of urllib handlers.
        """
        handlers = suds.transport.http.HttpTransport.u2handlers(self)
        if self.ssl_context:
            try:
                handlers.append(HTTPSHandler(context=self.ssl_context, check_hostname=self.verify))
            except TypeError:
                # Python 2.7.9 HTTPSHandler does not accept the
                # check_hostname keyword argument.
                #
                # Note that even older Python versions would also
                # croak on the context keyword argument.  But these
                # old versions do not have SSLContext either, so we
                # will not end up here in the first place.
                handlers.append(HTTPSHandler(context=self.ssl_context))
        return handlers


def get_https_transport(verify_ssl, cacert_file):

    ssl_context = None

    if verify_ssl == False and cacert_file == False:
        ssl_context = create_ssl_context(False, None, None)
    elif verify_ssl and not cacert_file:
        ssl_context = create_ssl_context(True, None, None)
    elif verify_ssl and cacert_file != False:
        ssl_context = create_ssl_context(True, cacert_file, None)

    return HTTPSTransport(ssl_context)


class PortList:
    """Represents a Deep Security Port List"""
    def __init__(self, id, name, description, ports, tbuid):
        self.id = id
        self.name = name
        self.description = description
        self.ports = ports
        self.tbuid = tbuid


from suds import Client

class TimeFilter:
    """Represents a Deep Security TimeFilter Transport"""
    def __init__(self, suds_client, rangeFrom=None, rangeTo=None, specificTime=None, time_type="LAST_HOUR"):
        self.rangeFrom = rangeFrom
        self.rangeTo = rangeTo
        self.specificTime = specificTime
        self.client = suds_client
        self.time_type = time_type

    def get_transport(self):
        tft = self.client.factory.create('TimeFilterTransport')
        tft.rangeFrom = self.rangeFrom
        tft.rangeTo = self.rangeTo
        tft.specificTime = self.specificTime

        etft = self.client.factory.create('EnumTimeFilterType')

        types = {"LAST_HOUR": etft.LAST_HOUR,
                 "LAST_24_HOURS": etft.LAST_24_HOURS,
                 "LAST_7_DAYS": etft.LAST_7_DAYS,
                 "CUSTOM_RANGE": etft.CUSTOM_RANGE,
                 "SPECIFIC_TIME": etft.SPECIFIC_TIME}

        if types[self.time_type]:
            tft.type = types[self.time_type]
        elif self.rangeFrom is not None and self.rangeTo is not None:
            tft.type = etft.CUSTOM_RANGE
        elif self.specificTime is not None:
            tft.type = etft.SPECIFIC_TIME
        else:
            tft.type = etft.LAST_HOUR

        return tft



"""
Created on Nov 3 2016
@author: Jeff Thorne
"""
import json
from datetime import datetime
import time
from typing import List, Dict
import urllib3
import ssl
import sys
import logging

from suds import Client, WebFault
import requests

from .host import HostFilter
from .idfilter import IDFilter
from .iplist import IPList
from .portlist import PortList
from .timefilter import TimeFilter
from ..utilities.cloudacct_utils import CloudAcctUtils
from..utilities.host_utils import HostUtils
from ..utilities import iplists as ipl_utils
from ..utilities import portlist_utils as pl_utils
from ..utilities.usages_utils import UsageUtils
from ..utilities.sslcontext import create_ssl_context, get_https_transport
from ..config import Config
from .modify_trusted_update_mode_request import ModifyTrustedUpdateModeRequest
from ..models.rest_objects import Scope, TimeRange, PropertyFilter, Scope, LiftApplicationDriftRequest, AddGlobalRulesetRulesRequest
from ..models.dpi_rule_transport import DPIRuleTransport




class Manager:

    def __init__(self, api_key: str = None, username: str = None, password: str = None, tenant=None, host: str ='app.deepsecurity.trendmicro.com',\
                 port: int = "443", verify_ssl:bool = False, cacert_file:str = False, api_version='v1'):
        """
        :param api_key   require to use some new rest calls. This calls will indicate api_key auth required in doc.
        :param username: required to use deprecated SOAP and rest calls
        :param password: required to use deprecated SOAP and rest calls
        :param tenant:   required to use deprecated SOAP and rest calls
        :param host:
        :param port:
        :param verify_ssl:
        :param cacert_file: optional CA certificates to trust for certificate verification
        """
        kwargs = {}

        self.api_version = api_version
        self.headers = {'Content-Type': 'application/json', 'api-version': self.api_version}

        if api_key is not None:
            self.api_key = api_key
            self.headers['api-secret-key'] = self.api_key

        self._username = username
        self._password = password
        self._tenant = tenant
        self.host = host
        self.port = port
        self.verify_ssl = verify_ssl
        self.config = Config(self.host, self.port)
        url = self.config.soap_url()
        urllib3.disable_warnings()

        kwargs['transport'] = get_https_transport(verify_ssl, cacert_file)

        try:
            self.client = Client(url, **kwargs)
        except ssl.CertificateError as ce:
            print(ce)
            sys.exit("could not verify ssl cert")

        try:
            if tenant and username is not None:
                self.session_id = self._authenticate_tenant()
            elif username is not None:
                self.session_id = self.__authenticate()
        except WebFault as detail:
            print("Authentication error: ", detail)
            sys.exit()

    def __authenticate(self) -> str:
        return self.client.service.authenticate(username=self._username, password=self._password)

    def authenticate_via_rest(self):
        dscrendentials = json.dumps(dict(dsCredentials=dict(userName=self._username, password=self._password)))
        url = "https://{}:{}/rest/authentication/login".format(self.host, self.port)
        headers = {'Content-Type': 'application/json'}
        r = requests.post(url, data=dscrendentials, verify=False, headers=headers)
        return r.content.decode('utf-8')

    def _authenticate_tenant(self):
        return self.client.service.authenticateTenant(tenantName=self._tenant, username=self._username, password=self._password)

    def get_api_version(self) -> int:
        """
        Retrieves the api version of Trend Micro's Deep Security SOAP Web Service.

        :return: int: The api version number.
        """
        return self.client.service.getApiVersion()

    def get_port_lists_all(self) -> List[PortList]:
        """
        Retrieves a list of all reusable post lists.

        :return: List[dsp3.models.portlist.PortList]
        """
        port_lists = self.client.service.portListRetrieveAll(sID=self.session_id)
        return pl_utils.parse_port_lists(port_lists)

    def get_ip_lists_all(self) -> List[IPList]:
        ip_lists =  self.client.service.IPListRetrieveAll(sID=self.session_id)
        return ipl_utils.parse_ip_lists(ip_lists)

    def save_ip_list(self, ip_list: IPList) -> Dict:
        iplto = ipl_utils.convert_to_tansport_ip_list(ip_list, self.client) #return IPListTransport object
        new_iplto = self.client.service.IPListSave(ipl=iplto, sID=self.session_id)
        if new_iplto:
            return new_iplto
        else:
            return "There was a problem"

    def ip_list_save(self,ip_list):
        return self.client.service.IPListSave(ipl=ip_list, sID=self.session_id)

    def get_ip_list(self, id):
        return self.client.service.IPListRetrieve(id, self.session_id)

    def get_ip_list_by_name(self, name):
        return self.client.service.IPListRetrieveByName(name, self.session_id)


    def delete_ip_list(self, ids):
        """
          Deletes the ip_list with the give id

          :param ids: The id(s) of the ip_list(s) to delete as a string.\
                      For a single id use a string and a list of string ids for multiple deletions
          :return: None
        """

        self.client.service.IPListDelete(sID=self.session_id, ids=ids)


    def get_cloudaccounts(self):
        """

        :return:
        """
        return CloudAcctUtils(self.config).get_cloudAccounts(self.session_id, self.verify_ssl)

    def get_cloudaccount(self, id):
        """

        :param id:
        :return:
        """
        return CloudAcctUtils(self.config).get_cloudAccount(id, self.session_id, self.verify_ssl)

    def cloudaccout_testconnection(self, id: str) -> Dict[str, str]:
        """

        :param id:
        :return:
        """
        return CloudAcctUtils(self.config).test_connection(id, self.session_id, self.verify_ssl)

    def cloudaccout_syncronize(self, id: str) -> Dict[str, str]:
        """

        :param id:
        :return:
        """
        return CloudAcctUtils(self.config).syncronize_account(id, self.session_id, self.verify_ssl)

    def get_jvmusage(self, manager_node_id:str = "", from_date: datetime = None, to_date: datetime = None) -> Dict[str, str]:
        """
        :param manager_node_id: ID of the manager node to retrieve usage info for. If not set, usage info for all manager nodes is retrieved.
        :param from_date: The date from which to list the usage statistics. If not set, then a time of one hour ago is used.
        :param to_date: The date up to which to gather the usage. If not set, the current time is used.
        :return: Dict[str, str] containing json virtual machine statistics.
        """
        return UsageUtils(self.config).jvm_usage(self.session_id, manager_node_id, from_date, to_date, self.verify_ssl)


    def get_host_by_name(self, name:str):
        """
        :param name:
        :return:
        """
        response = self.client.service.hostRetrieveByName(name, sID=self.session_id)
        if response == None:
            return None
        else:
            return HostUtils(self.config).create_host(response)

    def host_create(self, host_transport):
        return self.client.service.hostCreate(host=host_transport, sId=self.session_id)

    def host_detail_retrieve(self, host_group_id: int=None, host_id:int = None, security_profile_id:int = None,
                             host_type=None, host_detail_level: str ='HIGH'):
        """
        This function allows it, to get more information about hosts.
        (e.g. 'virtual Name' and 'virtual Uuid' of host)

        :param host_detail_level: options are: "LOW", "MEDIUM" and "HIGH"
        """

        host_filter = HostFilter(
            self.client, hostGroupId=host_group_id, host_id=host_id,
            securityProfileId=security_profile_id, type=host_type
        ).get_transport()

        hft = HostFilter(self.client, hostGroupId=host_group_id, host_id=host_id, securityProfileId=security_profile_id,
                         type=host_type).get_transport()


        response = self.client.service.hostDetailRetrieve(hostFilter=host_filter, hostDetailLevel=host_detail_level, sID=self.session_id)

        if isinstance(response, list) and len(response) == 1:
            return response[0]
        return response

    def host_status(self, id:int):
        """
        :param id: DS host id as string
        :return: suds.sudsobject.HostStatusTransport
        """
        return self.client.service.hostGetStatus(id, self.session_id)

    def host_move_to_hosts_group(self, host_ids, host_group_id):
        return self.client.service.hostMoveToHostGroup(host_ids, host_group_id, self.session_id)

    def host_agent_deactivate(self, ids:List[int]) -> None:
        """

        :param ids:
        :return:
        """
        self.client.service.hostAgentDeactivate(ids, self.session_id)



    def host_agent_activate(self, ids:List[int]) -> None:
        """

        :param ids:
        :return:
        """
        self.client.service.hostAgentActivate(ids, self.session_id)

    def host_components(self, host_id:str):
        """

        :param host_id:
        :return:
        """
        return HostUtils(self.config).components(host_id, self.session_id)


    def host_update_now(self, ids:List[int]) -> None:
        """

        :param ids:
        :return:
        """
        self.client.service.hostUpdateNow(ids, self.session_id)

    def host_getevents_now(self, ids:List[int]) -> None:
        """

        :param ids:
        :return:
        """
        self.client.service.hostGetEventsNow(ids, self.session_id)

    def host_getevents_nowsync(self, id:str) -> None:
        """

        :param id:
        :return:
        """
        self.client.service.hostGetEventsNowSync(id, self.session_id)


    def host_retrieve_by_hostgroup(self, host_group_id):
        """
        Retrieve hosts by host group.

        :param host_group_id: id of the host group.
        :return: List of HostTransport Objects. Example below:

                    (HostTransport){
                           ID = 1604
                           description = None
                           name = "ec2-184-72-238-128.compute-1.amazonaws.com"
                           displayName = "Ubuntu nginx Web Server"
                           external = True
                           externalID = None
                           hostGroupID = 432
                           hostType = "STANDARD"
                           platform = "Ubuntu Linux 12 (64 bit) (3.2.0-31-virtual)"
                           securityProfileID = 201
                    }
        """
        return self.client.service.hostRetrieveByHostGroup(host_group_id, self.session_id)

    def host_retrieve_all(self):
        """

        :return:
        """
        return self.client.service.hostRetrieveAll(self.session_id)


    def host_group_retrieve_all(self):
        """

        :return: List of HostGroupTransport objects. Example object below:
                        (HostGroupTransport){
                                                ID = 425
                                                description = None
                                                name = "vpc-7b3bd512"
                                                external = True
                                                externalID = None
                                                parentGroupID = 424
                                             }
        """
        return self.client.service.hostGroupRetrieveAll(self.session_id)



    def host_integrity_scan(self, ids:List[int]) -> None:
        """

        :param ids:
        :return:
        """
        self.client.service.hostIntegrityScan(ids, self.session_id)


    def dpi_rules_all(self):
        """

        :return:
        """
        return self.client.service.DPIRuleRetrieveAll(self.session_id)

    def host_group_retrieve_by_name(self, name):
        return self.client.service.hostGroupRetrieveByName(name, self.session_id)

    def host_group_delete(self, id):
        return self.client.service.hostGroupDelete(id, self.session_id)

    def host_group_retrieve_by_id(self, id):
        return self.client.service.hostGroupRetrieve(id, self.session_id)


    def host_group_create(self, name, description="", external=False, external_id=None, parent_group_id=None):
        """

        :param name:
        :return:
        """
        hgt = self.client.factory.create('HostGroupTransport')
        hgt['name'] = name
        hgt['description'] = description
        hgt['external'] = external
        hgt['externalID'] = external_id
        hgt['parentGroupID'] = parent_group_id
        return self.client.service.hostGroupCreate(hgt, self.session_id)


    def hostRetrieveByHostGroup(self, id):
        return self.client.service.hostRetrieveByHostGroup(id, self.session_id)


    def host_recommendation_scan(self, ids:List[int]):
        """
        This function runs a recomendation scan on an individual or list of hosts by id.

        :param ids: list of host ids to scan for reccomendations
        :return: None
        """
        response = self.client.service.hostRecommendationScan(ids, self.session_id)
        return response


    def antimalware_retreive_all(self):
        """

        :return:
        """
        return self.client.service.antiMalwareRetrieveAll(sID=self.session_id)



    def antimalware_event_retrieve(self, range_from=None, range_to=None, specific_time=None, time_type="LAST_HOUR",
                                   host_id=None, host_group_id=None, security_profile_id=None, host_type=None,
                                   event_id=1, event_operator="GREATER_THAN"):
        """
        This function retreives antimalware (AM) events from the Deep Security Manager based on several criteria specifice
        as paramaters. Several parameters are options.


        The first set of parameters are related to the time of event retrieval. All time parameters are optional and if not set
        time_type will default to "LAST_HOUR".

        :param range_from: retrieve events from this time. if range_from and range_to are set time_type is not required.
        :param range_to: retrieve events to this time
        :param specific_time: retieve event for a specific time. if specific_time isset time_type is not required.
        :param time_type: options are: "LAST_HOUR", "LAST_24_HOURS", "LAST_7_DAYS". if set range_from, range_to, and
                          specific time are not to be specified.


        The second set of parameters are related to the host/s AM event retreival is requested for. All host parameters
        are optional and if not set host_type will default to "ALL_HOSTS".

        :param host_id: host to retrieve events for. if set host_type defaults to "SPECIFIC_HOST"
        :param host_group_id: group to retreive events for. if set host_type defaults to "HOSTS_IN_GROUP_AND_ALL_SUBGROUPS"
        :param security_profile_id: security profile to retreive events for: if set host_type defaults to "HOSTS_USING_SECURITY_PROFILE"
        :param host_type: optional. options are "ALL_HOSTS", "HOSTS_IN_GROUP", "HOSTS_USING_SECURITY_PROFILE",
                 "HOSTS_IN_GROUP_AND_ALL_SUBGROUPS","SPECIFIC_HOST", "MY_HOSTS"


        These parameters are used as a search criteria to limit the scope of objects returned by event transport object ID
        :param event_id: Event transport objects ID to filter by. if not set will default to 1
        :param event_operator: options "GREATER_THAN", "LESS_THAN", "EQUAL". if not set will default to "GREATER_THAN"

        :return: None or [] of AntiMalwareEvent
        """

        response = None
        tft = TimeFilter(self.client, range_from, range_to, specific_time, time_type).get_transport()
        hft = HostFilter(self.client, hostGroupId=host_group_id, host_id=host_id, securityProfileId=security_profile_id, type=host_type).get_transport()
        idft = IDFilter(event_id, event_operator, self.client).get_transport()

        try:
            response = self.client.service.antiMalwareEventRetrieve(timeFilter=tft, hostFilter=hft, eventIdFilter=idft, sID=self.session_id)
            if response['antiMalwareEvents'] is None:
                return None

            return response['antiMalwareEvents']['item']

        except TypeError as te:
            logging.error(te, exc_info=True)
        except Exception as e:
            logging.error(e, exc_info=True)


    def webrep_event_retrieve(self, range_from=None, range_to=None, specific_time=None, time_type="LAST_HOUR",
                                   host_id=None, host_group_id=None, security_profile_id=None, host_type=None,
                                   event_id=1, event_operator="GREATER_THAN"):
        """
        This function retreives web reputation (WR) events from the Deep Security Manager based on several criteria specifice
        as paramaters. Several parameters are options.


        The first set of parameters are related to the time of event retrieval. All time parameters are optional and if not set
        time_type will default to "LAST_HOUR".

        :param range_from: retrieve events from this time. if range_from and range_to are set time_type is not required.
        :param range_to: retrieve events to this time
        :param specific_time: retieve event for a specific time. if specific_time isset time_type is not required.
        :param time_type: options are: "LAST_HOUR", "LAST_24_HOURS", "LAST_7_DAYS". if set range_from, range_to, and
                          specific time are not to be specified.


        The second set of parameters are related to the host/s AM event retreival is requested for. All host parameters
        are optional and if not set host_type will default to "ALL_HOSTS".

        :param host_id: host to retrieve events for. if set host_type defaults to "SPECIFIC_HOST"
        :param host_group_id: group to retreive events for. if set host_type defaults to "HOSTS_IN_GROUP_AND_ALL_SUBGROUPS"
        :param security_profile_id: security profile to retreive events for: if set host_type defaults to "HOSTS_USING_SECURITY_PROFILE"
        :param host_type: optional. options are "ALL_HOSTS", "HOSTS_IN_GROUP", "HOSTS_USING_SECURITY_PROFILE",
                 "HOSTS_IN_GROUP_AND_ALL_SUBGROUPS","SPECIFIC_HOST", "MY_HOSTS"


        These parameters are used as a search criteria to limit the scope of objects returned by event transport object ID
        :param event_id: Event transport objects ID to filter by. if not set will default to 1
        :param event_operator: options "GREATER_THAN", "LESS_THAN", "EQUAL". if not set will default to "GREATER_THAN"

        :return: None or [] of WebReputationEvent
        """

        response = None
        tft = TimeFilter(self.client, range_from, range_to, specific_time, time_type).get_transport()
        hft = HostFilter(self.client, hostGroupId=host_group_id, host_id=host_id, securityProfileId=security_profile_id, type=host_type).get_transport()
        idft = IDFilter(event_id, event_operator, self.client).get_transport()

        try:
            response = self.client.service.webReputationEventRetrieve(timeFilter=tft, hostFilter=hft, eventIdFilter=idft, sID=self.session_id)
            if response['webReputationEvents'] is None:
                return None


            return response['webReputationEvents']['item']

        except TypeError as te:
            logging.error(te, exc_info=True)
        except Exception as e:
            logging.error(e, exc_info=True)


    def fw_event_retrieve(self, range_from=None, range_to=None, specific_time=None, time_type="LAST_HOUR",
                                   host_id=None, host_group_id=None, security_profile_id=None, host_type=None,
                                   event_id=1, event_operator="GREATER_THAN"):
        """
        This function retrieves firewall (FW) events from the Deep Security Manager based on several criteria specifice
        as paramaters. Several parameters are options.


        The first set of parameters are related to the time of event retrieval. All time parameters are optional and if not set
        time_type will default to "LAST_HOUR".

        :param range_from: retrieve events from this time. if range_from and range_to are set time_type is not required.
        :param range_to: retrieve events to this time
        :param specific_time: retieve event for a specific time. if specific_time isset time_type is not required.
        :param time_type: options are: "LAST_HOUR", "LAST_24_HOURS", "LAST_7_DAYS". if set range_from, range_to, and
                          specific time are not to be specified.


        The second set of parameters are related to the host/s AM event retreival is requested for. All host parameters
        are optional and if not set host_type will default to "ALL_HOSTS".

        :param host_id: host to retrieve events for. if set host_type defaults to "SPECIFIC_HOST"
        :param host_group_id: group to retreive events for. if set host_type defaults to "HOSTS_IN_GROUP_AND_ALL_SUBGROUPS"
        :param security_profile_id: security profile to retreive events for: if set host_type defaults to "HOSTS_USING_SECURITY_PROFILE"
        :param host_type: optional. options are "ALL_HOSTS", "HOSTS_IN_GROUP", "HOSTS_USING_SECURITY_PROFILE",
                 "HOSTS_IN_GROUP_AND_ALL_SUBGROUPS","SPECIFIC_HOST", "MY_HOSTS"


        These parameters are used as a search criteria to limit the scope of objects returned by event transport object ID
        :param event_id: Event transport objects ID to filter by. if not set will default to 1
        :param event_operator: options "GREATER_THAN", "LESS_THAN", "EQUAL". if not set will default to "GREATER_THAN"

        :return: None or [] of FireWallEvent
        """

        response = None
        tft = TimeFilter(self.client, range_from, range_to, specific_time, time_type).get_transport()
        hft = HostFilter(self.client, hostGroupId=host_group_id, host_id=host_id, securityProfileId=security_profile_id, type=host_type).get_transport()
        idft = IDFilter(event_id, event_operator, self.client).get_transport()

        try:
            response = self.client.service.firewallEventRetrieve(timeFilter=tft, hostFilter=hft, eventIdFilter=idft, sID=self.session_id)
            if response['firewallEvents'] is None:
                return None

            return response['firewallEvents']['item']

        except TypeError as te:
            logging.error(te, exc_info=True)
        except Exception as e:
            logging.error(e, exc_info=True)


    def dpi_event_retrieve(self, range_from=None, range_to=None, specific_time=None, time_type="LAST_HOUR",
                                   host_id=None, host_group_id=None, security_profile_id=None, host_type=None,
                                   event_id=1, event_operator="GREATER_THAN"):
        """
        This function retrieves Deep Packet Inspection (DPI) events from the Deep Security Manager based on several criteria specifice
        as paramaters. Several parameters are options.


        The first set of parameters are related to the time of event retrieval. All time parameters are optional and if not set
        time_type will default to "LAST_HOUR".

        :param range_from: retrieve events from this time. if range_from and range_to are set time_type is not required.
        :param range_to: retrieve events to this time
        :param specific_time: retieve event for a specific time. if specific_time isset time_type is not required.
        :param time_type: options are: "LAST_HOUR", "LAST_24_HOURS", "LAST_7_DAYS". if set range_from, range_to, and
                          specific time are not to be specified.


        The second set of parameters are related to the host/s AM event retreival is requested for. All host parameters
        are optional and if not set host_type will default to "ALL_HOSTS".

        :param host_id: host to retrieve events for. if set host_type defaults to "SPECIFIC_HOST"
        :param host_group_id: group to retreive events for. if set host_type defaults to "HOSTS_IN_GROUP_AND_ALL_SUBGROUPS"
        :param security_profile_id: security profile to retreive events for: if set host_type defaults to "HOSTS_USING_SECURITY_PROFILE"
        :param host_type: optional. options are "ALL_HOSTS", "HOSTS_IN_GROUP", "HOSTS_USING_SECURITY_PROFILE",
                 "HOSTS_IN_GROUP_AND_ALL_SUBGROUPS","SPECIFIC_HOST", "MY_HOSTS"


        These parameters are used as a search criteria to limit the scope of objects returned by event transport object ID
        :param event_id: Event transport objects ID to filter by. if not set will default to 1
        :param event_operator: options "GREATER_THAN", "LESS_THAN", "EQUAL". if not set will default to "GREATER_THAN"

        :return: None or [] of DPIEventTransport
        """

        response = None
        tft = TimeFilter(self.client, range_from, range_to, specific_time, time_type).get_transport()
        hft = HostFilter(self.client, hostGroupId=host_group_id, host_id=host_id, securityProfileId=security_profile_id, type=host_type).get_transport()
        idft = IDFilter(event_id, event_operator, self.client).get_transport()

        try:
            response = self.client.service.DPIEventRetrieve(timeFilter=tft, hostFilter=hft, eventIdFilter=idft, sID=self.session_id)
            if response['DPIEvents'] is None:
                return None

            return response['DPIEvents']['item']

        except TypeError as te:
            logging.error(te, exc_info=True)
        except Exception as e:
            logging.error(e, exc_info=True)

    def im_event_retrieve(self, range_from=None, range_to=None, specific_time=None, time_type="LAST_HOUR",
                                   host_id=None, host_group_id=None, security_profile_id=None, host_type=None,
                                   event_id=1, event_operator="GREATER_THAN"):
        """
        This function retrieves integrity monitorinig (IM) events from the Deep Security Manager based on several criteria specifice
        as paramaters. Several parameters are options.


        The first set of parameters are related to the time of event retrieval. All time parameters are optional and if not set
        time_type will default to "LAST_HOUR".

        :param range_from: retrieve events from this time. if range_from and range_to are set time_type is not required.
        :param range_to: retrieve events to this time
        :param specific_time: retieve event for a specific time. if specific_time isset time_type is not required.
        :param time_type: options are: "LAST_HOUR", "LAST_24_HOURS", "LAST_7_DAYS". if set range_from, range_to, and
                          specific time are not to be specified.


        The second set of parameters are related to the host/s AM event retreival is requested for. All host parameters
        are optional and if not set host_type will default to "ALL_HOSTS".

        :param host_id: host to retrieve events for. if set host_type defaults to "SPECIFIC_HOST"
        :param host_group_id: group to retreive events for. if set host_type defaults to "HOSTS_IN_GROUP_AND_ALL_SUBGROUPS"
        :param security_profile_id: security profile to retreive events for: if set host_type defaults to "HOSTS_USING_SECURITY_PROFILE"
        :param host_type: optional. options are "ALL_HOSTS", "HOSTS_IN_GROUP", "HOSTS_USING_SECURITY_PROFILE",
                 "HOSTS_IN_GROUP_AND_ALL_SUBGROUPS","SPECIFIC_HOST", "MY_HOSTS"


        These parameters are used as a search criteria to limit the scope of objects returned by event transport object ID
        :param event_id: Event transport objects ID to filter by. if not set will default to 1
        :param event_operator: options "GREATER_THAN", "LESS_THAN", "EQUAL". if not set will default to "GREATER_THAN"

        :return: None or [] of IntegrityEventTransport
        """

        response = None
        tft = TimeFilter(self.client, range_from, range_to, specific_time, time_type).get_transport()
        hft = HostFilter(self.client, hostGroupId=host_group_id, host_id=host_id, securityProfileId=security_profile_id, type=host_type).get_transport()
        idft = IDFilter(event_id, event_operator, self.client).get_transport()

        try:
            response = self.client.service.integrityEventRetrieve(timeFilter=tft, hostFilter=hft, eventIdFilter=idft, sID=self.session_id)
            if response['integrityEvents'] is None:
                return None

            return response['integrityEvents']['item']

        except TypeError as te:
            logging.error(te, exc_info=True)
        except Exception as e:
            logging.error(e, exc_info=True)


    def li_event_retrieve(self, range_from=None, range_to=None, specific_time=None, time_type="LAST_HOUR",
                          host_id=None, host_group_id=None, security_profile_id=None, host_type=None,
                          event_id=1, event_operator="GREATER_THAN"):
        """
        This function retrieves log inspection (LI) events from the Deep Security Manager based on several criteria specifice
        as paramaters. Several parameters are options.


        The first set of parameters are related to the time of event retrieval. All time parameters are optional and if not set
        time_type will default to "LAST_HOUR".

        :param range_from: retrieve events from this time. if range_from and range_to are set time_type is not required.
        :param range_to: retrieve events to this time
        :param specific_time: retieve event for a specific time. if specific_time isset time_type is not required.
        :param time_type: options are: "LAST_HOUR", "LAST_24_HOURS", "LAST_7_DAYS". if set range_from, range_to, and
                          specific time are not to be specified.


        The second set of parameters are related to the host/s AM event retreival is requested for. All host parameters
        are optional and if not set host_type will default to "ALL_HOSTS".

        :param host_id: host to retrieve events for. if set host_type defaults to "SPECIFIC_HOST"
        :param host_group_id: group to retreive events for. if set host_type defaults to "HOSTS_IN_GROUP_AND_ALL_SUBGROUPS"
        :param security_profile_id: security profile to retreive events for: if set host_type defaults to "HOSTS_USING_SECURITY_PROFILE"
        :param host_type: optional. options are "ALL_HOSTS", "HOSTS_IN_GROUP", "HOSTS_USING_SECURITY_PROFILE",
                 "HOSTS_IN_GROUP_AND_ALL_SUBGROUPS","SPECIFIC_HOST", "MY_HOSTS"


        These parameters are used as a search criteria to limit the scope of objects returned by event transport object ID
        :param event_id: Event transport objects ID to filter by. if not set will default to 1
        :param event_operator: options "GREATER_THAN", "LESS_THAN", "EQUAL". if not set will default to "GREATER_THAN"

        :return: None or [] of LogInspectionEventTransport
        """

        response = None
        tft = TimeFilter(self.client, range_from, range_to, specific_time, time_type).get_transport()
        hft = HostFilter(self.client, hostGroupId=host_group_id, host_id=host_id, securityProfileId=security_profile_id,
                         type=host_type).get_transport()
        idft = IDFilter(event_id, event_operator, self.client).get_transport()

        try:
            response = self.client.service.logInspectionEventRetrieve(timeFilter=tft, hostFilter=hft, eventIdFilter=idft,
                                                                  sID=self.session_id)

            if response['logInspectionEvents'] is None:
                return None

            return response['logInspectionEvents']['item']

        except TypeError as te:
            logging.error(te, exc_info=True)
        except Exception as e:
            logging.error(e, exc_info=True)




    def antimailware_retrieve_by_name(self, name):
        """
        This function retrieves the AntiMalware with the provided name (Case sensitive)

        :param name: The name of the AntiMalware to retrieve which is case sensitive
        :return: AntiMalwareTransport object.
        """
        response = self.client.service.antiMalwareRetrieveByName(name, sID=self.session_id)
        return response


    def system_event_retrieve(self, range_from=None, range_to=None, specific_time=None, time_type="LAST_HOUR",
                              host_id=None, host_group_id=None, security_profile_id=None, host_type=None,
                              event_id=1, event_operator="GREATER_THAN", includeNonHostEvents=True):
        """
               This function retrieves system events from the Deep Security Manager based on several criteria specifice
               as paramaters. Several parameters are options.


               The first set of parameters are related to the time of event retrieval. All time parameters are optional and if not set
               time_type will default to "LAST_HOUR".

               :param range_from: retrieve events from this time. if range_from and range_to are set time_type is not required.
               :param range_to: retrieve events to this time
               :param specific_time: retieve event for a specific time. if specific_time isset time_type is not required.
               :param time_type: options are: "LAST_HOUR", "LAST_24_HOURS", "LAST_7_DAYS". if set range_from, range_to, and
                                 specific time are not to be specified.


               The second set of parameters are related to the host/s AM event retreival is requested for. All host parameters
               are optional and if not set host_type will default to "ALL_HOSTS".

               :param host_id: host to retrieve events for. if set host_type defaults to "SPECIFIC_HOST"
               :param host_group_id: group to retreive events for. if set host_type defaults to "HOSTS_IN_GROUP_AND_ALL_SUBGROUPS"
               :param security_profile_id: security profile to retreive events for: if set host_type defaults to "HOSTS_USING_SECURITY_PROFILE"
               :param host_type: optional. options are "ALL_HOSTS", "HOSTS_IN_GROUP", "HOSTS_USING_SECURITY_PROFILE",
                        "HOSTS_IN_GROUP_AND_ALL_SUBGROUPS","SPECIFIC_HOST", "MY_HOSTS"


               These parameters are used as a search criteria to limit the scope of objects returned by event transport object ID
               :param event_id: Event transport objects ID to filter by. if not set will default to 1
               :param event_operator: options "GREATER_THAN", "LESS_THAN", "EQUAL". if not set will default to "GREATER_THAN"

               :return: None or [] of SystemEventTransport
         """

        response = None
        tft = TimeFilter(self.client, range_from, range_to, specific_time, time_type).get_transport()
        hft = HostFilter(self.client, hostGroupId=host_group_id, host_id=host_id, securityProfileId=security_profile_id,
                         type=host_type).get_transport()
        idft = IDFilter(event_id, event_operator, self.client).get_transport()

        try:
            response = self.client.service.systemEventRetrieve(timeFilter=tft, hostFilter=hft, eventIdFilter=idft,
                                                               sID=self.session_id, includeNonHostEvents=includeNonHostEvents)


            if response['systemEvents'] is None:
                return None

            return response['systemEvents']['item']

        except TypeError as te:
            logging.error(te, exc_info=True)
        except Exception as e:
            logging.error(e, exc_info=True)

    def set_trusted_update_mode(self, host_id: int, duration:int = 0, enabled: bool = True) -> str:
        """
        This function sets the trusted (maintenance) mode status of the host specified for a specific duration.
        NOTE: This call only supported in DS10 and higher

        :param host_id: host to enable or disable trusted (maintenance) mode for
        :param duration: the amount of time to enable trusted mode. Not required for disable request
        :param enabled: True to enable or False to disable trusted mode
        :return: status code
        """
        modify_trusted_updatemode_request = ModifyTrustedUpdateModeRequest(duration, enabled)
        url = "https://{}:{}/rest/hosts/{}/trusted-update-mode".format(self.host, self.port, host_id)
        headers = {'Content-Type': 'application/json'}
        cookies = dict(sID=self.session_id)
        r = requests.post(url, data=modify_trusted_updatemode_request.to_json(), verify=self.verify_ssl, cookies=cookies, headers=headers)
        return json.dumps(dict(status_code=r.status_code))

    def get_trusted_update_mode(self, host_id: int) -> str:
        """
        This function retreives the trusted (maintenance) mode status of the host specified.
        NOTE: This call only supported in DS10 and higher

        :param host_id: the id of the host to retreive trust update mode (maintenance) status on
        :return: json string of the format
                {  "DescribeTrustedUpdateModeResponse":
                    {
                      "startTimeHuman":"Sunday Jan 29 18:00:17 PM EST",
                      "endTimeHuman":"Sunday Jan 29 18:10:17 PM EST",
                      "state":"on",
                      "startTime":1485730817728,
                      "endTime":1485731417728
                   }
                }
        """
        url = "https://{}:{}/rest/hosts/{}/trusted-update-mode".format(self.host, self.port, host_id)
        headers = {'Content-Type': 'application/json'}
        cookies = dict(sID=self.session_id)
        r = requests.get(url, verify=self.verify_ssl, cookies=cookies, headers=headers)
        response = json.loads(r.content.decode('utf-8'))
        state = response['DescribeTrustedUpdateModeResponse']['state']

        if state != "off":
            start_time = response['DescribeTrustedUpdateModeResponse']['startTime']
            end_time = response['DescribeTrustedUpdateModeResponse']['endTime']
            human_start_time = time.strftime("%A %b %d %-H:%M:%S %p %Z", time.localtime(start_time / 1000.0)) if start_time != None else None
            human_end_time = time.strftime("%A %b %d %-H:%M:%S %p %Z", time.localtime(end_time / 1000.0)) if end_time != None else None

            return json.dumps(dict(DescribeTrustedUpdateModeResponse=dict(startTime=start_time, endTime=end_time, state=state, \
                                                                      endTimeHuman=human_end_time, startTimeHuman=human_start_time )))
        else:
            return json.dumps(dict(DescribeTrustedUpdateModeResponse=dict(state=state)))

    def decision_logs(self) -> Dict[str, str]:
        url = "https://{}:{}/rest/decision-logs".format(self.host, self.port)
        r = requests.get(url=url, verify=self.verify_ssl, cookies=dict(sID=self.session_id), headers=self.headers)
        return r.content.decode('utf-8')

    def decision_log(self, decision_log_id:int) -> Dict[str, str]:
        url = "https://{}:{}/rest/decision-logs/{}".format(self.host, self.port, decision_log_id)
        r = requests.get(url=url, verify=self.verify_ssl, cookies=dict(sID=self.session_id), headers=self.headers)
        return r.content.decode('utf-8')

    def decision_log_details(self, decision_log_id:int, start_id:int = 1, count:int = 1) -> Dict[str, str]:
        params = {'startID': start_id, 'count': count}
        url = "https://{}:{}/rest/decision-logs/{}/details".format(self.host, self.port, decision_log_id)
        r = requests.get(url=url, verify=self.verify_ssl, cookies=dict(sID=self.session_id), headers=self.headers, params=params)
        return r.content.decode('utf-8')



    def appcontrol_events(self, event_time: datetime = None, event_time_op:str = None, max_items: int = None) -> Dict[str, str]:
        """
        TODO: IMplement eventID and eventIDOp parameters
        NOTE: This call only supported in DS10 and higher

        :param event_time:  the event time to query for events.
        :param event_time_op: gt(greater than), ge(greater than or equal to), eq(eqaul to), lt(less than), and le(less than or equal to).
                              If an unsupported operator is provided, the default is 'eq'.
        :param max_items:  the maximum number of events to return
        :return: ListEventsResponse json dictionary
        """
        url = "https://{}:{}/rest/events/appcontrol".format(self.host, self.port)
        params = {'eventTime': event_time, 'eventTimeOp':event_time_op, 'maxItems': max_items }
        params['eventTime'] = self._convert_date(event_time) if event_time != None else None   #convert event_time to ms since epoch timestamp
        params = dict((k,v) for k,v in params.items() if v is not None)
        r = requests.get(url, verify=self.verify_ssl, cookies=dict(sID=self.session_id), headers=self.headers, params=params)
        return json.loads(r.content.decode('utf-8'))

    """ Seems to have been removed from API
    def license_info(self, module):
        url = "https://{}:{}/rest/license/listLicenseInfo".format(self.host, self.port)
        params = {'module': module, 'sID': self.session_id}
        r = requests.get(url, verify=self.verify_ssl, cookies=dict(sID=self.session_id), headers=self.headers, params=params)
        return json.loads(r.content.decode('utf-8'))

    """

    def manager_info_version(self) -> str:
        """
        Retrieve DSM version.

        :return: str representation of DSM version
        """
        url = "https://{}:{}/rest/managerInfo/version".format(self.host, self.port)
        r = requests.get(url, verify=self.verify_ssl, cookies=dict(sID=self.session_id), headers=self.headers)
        return r.content.decode('utf-8')

    def manager_info_status_summary(self) -> dict:
        """
        Retrieves the status summary of the system

        :return: StatusSummaryElement
        """
        url = "https://{}:{}/rest/managerInfo/statusSummary".format(self.host, self.port)
        params = {'sID': self.session_id}
        r = requests.get(url, verify=self.verify_ssl, cookies=dict(sID=self.session_id), headers=self.headers, params=params)
        return json.loads(r.content.decode('utf-8'))

    def manager_info_components(self) -> dict:
        """
        Retrieves detailed component info in current system

        :return: ComponentInfoElement
        """
        url = "https://{}:{}/rest/managerInfo/components".format(self.host, self.port)
        params = {'sID': self.session_id}
        r = requests.get(url, verify=self.verify_ssl, cookies=dict(sID=self.session_id), headers=self.headers, params=params)
        return json.loads(r.content.decode('utf-8'))

    def manager_info_feature_summary(self, timescale: int) -> dict:
        """
        Retrieves the status summary of each protection feature

        :param timescale 1 [last 7 days] or 2 [last 24 hours]
        :return: FeatureSummaryElement
        """
        url = "https://{}:{}/rest/managerInfo/featureSummary".format(self.host, self.port)
        params = {'sID': self.session_id, 'timescale': timescale}
        r = requests.get(url, verify=self.verify_ssl, cookies=dict(sID=self.session_id), headers=self.headers, params=params)
        return json.loads(r.content.decode('utf-8'))


    def alerts(self, alert_id:int=None, dismissed:bool=None, maxItems:int=None, op:str=None) -> dict:
        """
        alerts retrieves alert information from the dsm

        :param alert_id:  (optional) used to define the starting point for the query. Combine with op to page through results.
        :param dismissed: (optional) include alerts that have been dismissed.
        :param maxItems:  (optional) the number of items to retrieve.
        :param op:        (optional, required if alertID is specified) Currently supported operations are: gt, ge, eq, lt,le

        :return:   ListAlertsResponse
        """
        url = "https://{}:{}/rest/alerts".format(self.host, self.port)
        params = {'alertID': alert_id, 'dismised': dismissed, 'maxItems': maxItems, 'op':op}
        r = requests.get(url, verify=self.verify_ssl, cookies=dict(sID=self.session_id), headers=self.headers, params=params)
        return json.loads(r.content.decode('utf-8'))

    def appcontrol_event(self, event_id:int) -> Dict[str, str]:
        """
        Get the Application Control event with the specified event ID.

        :param event_id: the event ID
        :return: DescribeEventResponse json dict. containing the event with the specific ID
        """
        url = "https://{}:{}/rest/events/appcontrol/{}".format(self.host, self.port, event_id)
        r = requests.get(url, verify=self.verify_ssl, cookies=dict(sID=self.session_id), headers=self.headers)
        return json.loads(r.content.decode('utf-8'))

    def drift_applications(self, host_id:int, start_time: datetime, end_time:datetime, file_name:str, host_name:str):
        time_range = TimeRange(end_time, start_time)
        property_filter = PropertyFilter(file_name, host_name)
        scope = Scope(property_filter, time_range)
        lar = LiftApplicationDriftRequest(scope)
        url = "https://{}:{}/rest/software-inventory/drift/applications".format(self.host, self.port)
        r = requests.post(url, data=lar.to_json(), verify=self.verify_ssl, cookies=dict(sID=self.session_id), headers=self.headers)
        return json.loads(r.content.decode('utf-8'))



    def list_block_by_hash_rules(self):
        """
        NOTE: This call only works with DSM's > 10.2

        :return: Listing of existing Block by Hash Rules from Global Ruleset

        """
        url = "https://{}:{}/rest/rulesets/global".format(self.host, self.port)
        r = requests.get(url, verify=self.verify_ssl, cookies=dict(sID=self.session_id), headers=self.headers)
        return json.loads(r.content)

    def add_block_by_hash_rule(self, hash, description):
        """
        NOTE: This call only works with DSM's > 10.2
        This methods allows for the Adding of a new Block by Hash Rules to Global Ruleset.

        :param hash: the sha256 hash of the file to bock
        :param description: description of new BLock by Hash Rule
        :return: rule that was successfully added along with its corresponding ruleID
        """
        url = "https://{}:{}/rest/rulesets/global/rules".format(self.host, self.port)
        rule_request = AddGlobalRulesetRulesRequest(hash, description)
        r = requests.post(url, data=rule_request.to_json(), verify=self.verify_ssl, cookies=dict(sID=self.session_id),headers=self.headers)
        return json.loads(r.content)

    def delete_block_by_hash_rule(self, rule_id):
        """
        NOTE: This call only works with DSM's > 10.2
        This method provides for deleting an existing Block by Hash Rules to Global Ruleset.

        :param rule_id: The id of the Block by Hash rule to delete
        :return: response payload
        """
        url = "https://{}:{}/rest/rulesets/global/rules/{}".format(self.host, self.port, rule_id)
        r = requests.delete(url, verify=self.verify_ssl, cookies=dict(sID=self.session_id), headers=self.headers)
        return r



    def add_aws_cloud_account_with_keys(self, access_key, secret_key):
        return CloudAcctUtils(self.config).add_cloudaccount_aws(access_key, secret_key, self.session_id)

    def add_aws_cloud_account_with_cross_account_role(self, external_id, role_arn):
        return CloudAcctUtils(self.config).add_cloudaccount_aws_cross_account(external_id, role_arn, self.session_id)

    def security_profile_assign_to_host(self, securityid: int, hostid: int) -> None:
        """
        :param securityid: security policy id
        :param hostid: host id
        :return:
        """
        self.client.service.securityProfileAssignToHost(securityid, hostid, self.session_id)

    def host_delete(self, ids):
        return self.client.service.hostDelete(ids, self.session_id)

    def dpi_rule_save(self, application_type, name, eventOnPacketDrop, eventOnPacketModify, templateType, patternAction,
                patternIf, priority, signatureAction, severity, ruleXML, detectOnly=False, disableEvent=False,
                 ignoreRecommendations=False, includePacketData=False, patternCaseSensitive=False, raiseAlert=False,
                 signatureCaseSensitive=False, cvssScore=0, authoritative=False):

        rule = self.dpi_rule_retrieve_by_name(name)
        rule_id = None
        app_id = self.application_type_retreive_by_name(application_type).ID

        if rule and rule.ID is not None:
            rule_id = rule.ID
            #return '{"response": "A rule with that name already exists"}'


        dpirt = DPIRuleTransport(self.client, name, app_id, eventOnPacketDrop, eventOnPacketModify, templateType, patternAction,
                patternIf, priority, signatureAction, severity, ruleXML, rule_id, detectOnly, disableEvent,
                 ignoreRecommendations, includePacketData, patternCaseSensitive, raiseAlert,
                 signatureCaseSensitive, cvssScore, authoritative)

        return self.client.service.DPIRuleSave(ipsf=dpirt.get_transport(), sID=self.session_id)


    def dpi_rule_retrieve_by_name(self, name):
        return self.client.service.DPIRuleRetrieveByName(name, self.session_id)

    def dpi_rule_retrieve_by_id(self, id):
        """
        Retrieves info on a DPI rule by rule id

        :param id: dpi rule id
        :return: suds.sudsobject.DPIRuleTransport
        """
        return self.client.service.DPIRuleRetrieve(id, self.session_id)


    def fw_rule_retrieve_by_id(self, id):
        """
        Retrieves info on a FW rule by rule id

        :param id: fw rule id
        :return: suds.sudsobject.FirewallRuleTransport
        """
        return self.client.service.firewallRuleRetrieve(id, self.session_id)


    def fw_rule_save(self, fw_rule):
        """

        :param fw_rule: FirewallRuleTransport object to create or save
        :return: Newly created FirewallRuleTransport object.
        """
        return self.client.service.firewallRuleSave(fw_rule, self.session_id)


    def security_profile_save(self, security_profile_transport_object):
        """
        :param security_profile_transport_object: suds.sudsobject.SecurityProfileTransport
        :return: suds.sudsobject.SecurityProfileTransport
        """
        return self.client.service.securityProfileSave(sp=security_profile_transport_object, sID=self.session_id)


    def host_reccommendation_rule_ids_retrieve(self, host_id, rule_type=1, only_unassigned=False):
        """
        :param host_id:
        :param rule_type: 1=Intrusion Prevention application type rule, 2=Intrusion Prevention inspection rule, 4=Integrity Monitoring rule, 5=Log Inspection rule)
        :param only_unassigned:
        :return: list of rule ids
        """
        return self.client.service.hostRecommendationRuleIDsRetrieve(hostID=host_id, type=rule_type, onlyunassigned=only_unassigned, sID=self.session_id)

    def security_profile_reccommendation_rule_ids_retrieve(self, profile_id, rule_type=1):
        """
        :param profile_id: security policy id
        :param rule_type: rule_type: 1=Intrusion Prevention application type rule, 2=Intrusion Prevention inspection rule, 4=Integrity Monitoring rule, 5=Log Inspection rule
        :return: list of rule ids
        """
        return self.client.service.securityProfileRecommendationRuleIDsRetrieve(securityProfileID=profile_id, type=rule_type, sID=self.session_id)



    def end_session(self) -> None:
        """
        :return:
        """
        self.client.service.endSession(sID=self.session_id)

    def is_instance_protected_by_malware(self, host_name):
        pass


    def get_security_profile(self, id: int):
        """
        :param id: security policy id
        :return: suds.sudsobject.SecurityProfileTransport
        """
        return self.client.service.securityProfileRetrieve(id, self.session_id)

    def get_security_profile_by_name(self, name):
        """
        :param name: security policy name
        :return: suds.sudsobject.SecurityProfileTransport
        """
        return self.client.service.securityProfileRetrieveByName(name=name, sID=self.session_id)


    def host_clear_warnings_and_errors(self, hosts):
        """

        :param hosts: int if single host or list[int] if many hosts
        :return:
        """
        return self.client.service.hostClearWarningsErrors(hostIDs=hosts, sID=self.session_id)

    def application_type_retreive_by_name(self, name):
        return self.client.service.applicationTypeRetrieveByName(name, self.session_id)

    def software_retrieve_all(self):
        return self.client.service.softwareRetrieveAll(self.session_id)

    def administrators(self, admin_id:int=None, admin_op:str=None, max_items:int=None) -> Dict[str, str]:
        """
        administrators lists administrators.

        :param admin_id used to define the starting point for the query. Combine with administratorIDOp.
        :params admin_op required if administratorID is specified. gt, ge, eq, lt,le
        :return: ListAdministratorsResponse json
        """
        params = {'administratorID':admin_id, 'administratorIDOp': admin_op, 'maxItems': max_items}
        url = "https://{}:{}/rest/administrators".format(self.host, self.port)
        r = requests.get(url=url, verify=self.verify_ssl, cookies=dict(sID=self.session_id), headers=self.headers, params=params)
        return json.loads(r.content.decode('utf-8'))


    def event_based(self) -> dict:
        """
        List event-based tasks.

        :return: ListEventBasedTasksResponse json object
        """
        url = "https://{}:{}/rest/tasks/event-based".format(self.host, self.port)
        r = requests.get(url=url, verify=self.verify_ssl, cookies=dict(sID=self.session_id), headers=self.headers)
        return json.loads(r.content.decode('utf-8'))


    def event_based_delete(self, id:int) -> int:
        """
        Delete an event-based task.

        :param id: id of event based task
        :return: http status code
        """
        url = "https://{}:{}/rest/tasks/event-based/{}".format(self.host, self.port, id)
        r = requests.delete(url=url, verify=self.verify_ssl, cookies=dict(sID=self.session_id), headers=self.headers)
        return r.status_code

    def event_based_task_create(self, name:str, conditions:List[dict], actions:List[dict], task_type:str='computer-created-by-system',
                                enabled:bool=True) -> dict:
        """

        :param name:
        :param conditions list of dicts {field:'', key:'', value:''}
               field value one of: hostnameMatch, vcenterMatch, cloudProviderMatch, securityGroupMatch, imageIdMatch,
               esxMatch,folderMatch,platformMatch, applianceProtectionAvailable True or False,
               applianceProtectionActivated True or False, lastUsedIP, tagMatch, nsxSecurityGroupMatch
        :param actions List of dicts {'type':'', 'parameterValue':''}
               type value one of: activate, assign-policy, assign-relay, assign-group, deactivate
        :param type: one of: computer-created-by-system, agent-initiated-activation, agent-ip-changed, nsx-protection-changed,
                     computer-powered-on-by-system
        :param enabled the enabled state for this task.

        :return: CreateEventBasedTaskResponse
        """
        event_task = dict(name=name, type=task_type, enabled=enabled, conditions=conditions, actions=actions)
        task_request = dict(CreateEventBasedTaskRequest=dict(task=event_task))
        json_task = json.dumps(task_request)
        url = "https://{}:{}/rest/tasks/event-based".format(self.host, self.port)
        headers = {'Content-Type': 'application/json'}
        r = requests.post(url, data=json_task, verify=False, cookies=dict(sID=self.session_id), headers=headers)
        return r.content.decode('utf-8')



    def _convert_date(self, date:datetime) -> float:
        epoch = datetime.utcfromtimestamp(0)
        timestamp = (date - epoch).total_seconds() * 1000
        return int(timestamp)



    def list_relays(self, ascending:bool=None, background:bool=False, failed:bool=False, max_items:int=None, offset:int=None,
                    sort_by:str=None):
        """
        List relays
        :param ascending:  (optional) set true indicate ascending. Default is true. This parameter only works with sortBy.
        :param background: (optional) If true, does not extends the session. Default false.
        :param failed: (optional) set true, indicate that the API only returns the failure records of enabling/disabling.
                        If false, the API returns valid relays according to the specified criteria. Default is false.
        :param maxItems: (optional) the number of items to retrieve. The maximum value for this parameter is controlled
                         by the "Maximum number of items to retrieve from database" setting on the administrator account,
                         which defaults to 5000.
        :param offset: (optional) used to define the starting point for the query. This parameter only works with sortBy.
        :param sort_by: (optional) used to define the sorting field. The only available sorting column is Name. However,
                        if sortBy is not specified, the default sorting column is id. This parameter can work with maxItems,
                        ascending and offset.
        :return: ListRelaysResponse a ListRelaysResponse with the host details.
        """
        url = "https://{}:{}/rest/relays".format(self.host, self.port)
        params = dict(ascending=ascending, background=background, failed=failed, maxItems=max_items, offset=offset, sortBy=sort_by)
        r = requests.get(url=url, verify=self.verify_ssl, cookies=dict(sID=self.session_id), params=params, headers=self.headers)
        return json.dumps(r.content.decode('utf-8'))



    def scripts(self, id:int=None, max_items:int=None, op:str=None):
        """

        :param id: (optional) used to define the starting point for the query. Combine with op to page through results.
        :param max_integers:

        :param op: (optional, required if id is specified) Currently supported operations are:
                    gt (greater than), ge (greater than or equal to), eq (equal to), lt (less than),
                    le (less than or equal to)
        :return: ListScriptsResponse with the list of scripts.

        """
        url = "https://{}:{}/rest/scripts".format(self.host, self.port)
        params = dict(id=id, maxItems=max_items, op=op)
        r = requests.get(url=url, verify=self.verify_ssl, cookies=dict(sID=self.session_id), params=params,
                         headers=self.headers)
        return json.dumps(r.content.decode('utf-8'))


    def reports(self, id:int=None, max_items:int=None, op:str=None):
        """
        List report templates.
          :param id: (optional) used to define the starting point for the query. Combine with op to page through results.
          :param max_integers:

          :param op: (optional, required if id is specified) Currently supported operations are:
                      gt (greater than), ge (greater than or equal to), eq (equal to), lt (less than),
                      le (less than or equal to)
          :return: ListReportTemplatesResponse with the report template details.

          """
        url = "https://{}:{}/rest/reports".format(self.host, self.port)
        params = dict(id=id, maxItems=max_items, op=op)
        r = requests.get(url=url, verify=self.verify_ssl, cookies=dict(sID=self.session_id), params=params,
                         headers=self.headers)
        return json.dumps(r.content.decode('utf-8'))


    def computer_describe(self, host_id:int):
        url = "https://{}:{}/api/computers/{}".format(self.host, self.port, host_id)
        self.headers['api-version'] = 'v11.2.88'
        r = requests.get(url=url, verify=self.verify_ssl, cookies=dict(sID=self.session_id),headers=self.headers)
        return json.dumps(r.content.decode('utf-8'))


    def api_keys(self):
        '''
        api_key auth required to use this call

        :return: json object listing all api key info
        '''
        url = "https://{}:{}/api/apikeys".format(self.host, self.port)
        r = requests.get(url=url, verify=self.verify_ssl, headers=self.headers)
        return json.loads(r.content.decode('utf-8'))



    def tenants(self):
        '''
            api_key auth required to use this call

            :return: json object listing tenants
        '''
        url = "https://{}:{}/api/tenants".format(self.host, self.port)
        r = requests.get(url=url, verify=self.verify_ssl, headers=self.headers)
        return json.loads(r.content.decode('utf-8'))


    def create_tenant(self, database_server_id:int, name:str, description: str, agent_initated_activation_password:str,
                      time_zone:str,locale:str, modules_visible:List[str], hide_unlicensed_modules:bool, last_signin_time: int,
                      tenant_state: str, activation_codes:List[str], username:str, password: str, full_name: str, admin_description: str, role_id: int,
                      admin_locale:str, admin_time_zone: str, time_format: str, password_never_expires: bool, active: bool,
                      mfa_type: str, phone_number: str, mobile_number: str, pager_number: str, email_address: str,
                      primary_contact:bool, receive_notifications:bool, report_pdf_password_enabled: bool,
                      report_pdf_password:str, utf_offset: str, bypass_tenant_cache: bool = False, confirmation_required:bool = False,
                      asynchronous:bool=False, demo_mode:bool=False):


        url = "https://{}:{}/api/tenants?bypassTenantCache={}&confirmationRequired={}&asynchronous={}".format(self.host, self.port, bypass_tenant_cache, confirmation_required, asynchronous)
        administrator = dict(username=username, password=password, fullName=full_name, description=admin_description, roleID=role_id,
                             locale=admin_locale, timeZone=admin_time_zone, timeFormat=time_format, passwordNeverExpires=password_never_expires,
                             active=active, mfaType=mfa_type, phoneNumber=phone_number, mobileNumber=mobile_number,
                             pagerNumber=pager_number, emailAddress=email_address, primaryContact=primary_contact,
                             receiveNotifications=receive_notifications, reportPDFPasswordEnabled=report_pdf_password_enabled,
                             reportPDFPassword=report_pdf_password, UTCOffset=utf_offset)
        data = dict(databaseServerID=database_server_id, name=name, description=description, agentInitiatedActivationPassword=agent_initated_activation_password,
                    timeZone=time_zone, locale=locale, demoMode=demo_mode, modulesVisible=modules_visible, hideUnlicensedModules=hide_unlicensed_modules,
                    lastSigninTime=last_signin_time, tenantState=tenant_state, activationCodes=activation_codes, administrator=administrator)
        url = "https://{}:{}/api/tenants".format(self.host, self.port)
        print(json.dumps(data))
        r = requests.post(url, data=json.dumps(data), verify=False, headers=self.headers)
        return r.content.decode('utf-8')





from setuptools import setup

setup(name='dsp3',
      version='0.1c36',
      description='Python 3 client for Trend Micro\'s Deep Security Platform',
      url='https://github.com/trend206/dsp3',
      author='Jeff Thorne',
      author_email='jthorne@u.washington.edu',
      license='MIT',
      packages=['dsp3', 'dsp3.models', 'dsp3.utilities'],
      install_requires=['suds-py3 >= 1.2.0.0', 'requests >= 2.9.1'],
      zip_safe=False)


from typing import List

from ..models.iplist import IPList
from suds import Client


def parse_ip_lists(ip_lists) -> List[IPList]:
    """
    Used to turn a list if IPListTransport objects received by DSM into IPList objects

    :param ip_lists: IPListTransport
    :return: List[IPList]
    """
    iplists = []

    for ip_list in ip_lists:
        ips = ip_list.items.split("\n")
        ipl = IPList(ip_list.ID, ip_list.name, ip_list.description, ips)
        iplists.append(ipl)

    return iplists




def convert_to_tansport_ip_list(ip_list:IPList, suds_client:Client):
    iplt = suds_client.factory.create('IPListTransport')
    iplt.ID = ip_list.id
    iplt.description = ip_list.description
    iplt.name = ip_list.name
    iplt.items = "\n".join(item for item in ip_list.ips)
    return iplt


class DPIRuleTransport:

    def __init__(self, suds_client, name, applicationTypeID, eventOnPacketDrop, eventOnPacketModify, templateType, patternAction,
                patternIf, priority, signatureAction, severity, ruleXML, rule_id=   None, detectOnly=False, disableEvent=False,
                 ignoreRecommendations=False, includePacketData=False, patternCaseSensitive=False, raiseAlert=False,
                 signatureCaseSensitive=False, cvssScore=0, authoritative = False):

        self.client = suds_client
        self.name = name
        self.applicationTypeID = applicationTypeID
        self.eventOnPacketDrop = eventOnPacketDrop
        self.eventOnPacketModify = eventOnPacketModify
        self.templateType = templateType
        self.patternAction = patternAction
        self.patternIf = patternIf
        self.priority = priority
        self.signatureAction = signatureAction
        self.severity = severity
        self.ruleXML = ruleXML
        self.detectOnly = detectOnly
        self.disableEvent = disableEvent
        self.ignoreRecommendations = ignoreRecommendations
        self.includePacketData = includePacketData
        self.patternCaseSensitive = patternCaseSensitive
        self.signatureCaseSensitive = signatureCaseSensitive
        self.cvssScore = cvssScore
        self.authoritative = authoritative
        self.raiseAlert = raiseAlert

        if rule_id is not None:
            self.ID = rule_id

    def get_transport(self):
        dpirt = self.client.factory.create('DPIRuleTransport')
        dpirt.name = self.name
        dpirt.applicationTypeID = self.applicationTypeID
        dpirt.eventOnPacketDrop = self.eventOnPacketDrop
        dpirt.eventOnPacketModify = self.eventOnPacketModify
        dpirt.templateType = self.templateType
        dpirt.patternAction = self.patternAction
        dpirt.patternIf = self.patternIf
        dpirt.priority = self.priority
        dpirt.signatureAction = self.signatureAction
        dpirt.severity = self.severity
        dpirt.ruleXML = self.ruleXML
        dpirt.detectOnly = self.detectOnly
        dpirt.disableEvent = self.disableEvent
        dpirt.ignoreRecommendations = self.ignoreRecommendations
        dpirt.includePacketData = self.includePacketData
        dpirt.patternCaseSensitive = self.patternCaseSensitive
        dpirt.raiseAlert = self.raiseAlert
        dpirt.signatureCaseSensitive = self.signatureCaseSensitive
        dpirt.cvssScore = self.cvssScore
        dpirt.authoritative = self.authoritative

        if hasattr(self, 'ID'):
            dpirt.ID = self.ID

        return dpirt


import json
from typing import Dict


from ..config import Config
import requests


class CloudAcctUtils:


    def __init__(self, config):
        self.config = config
        self.headers = {'Accept': 'application/json'}
        self.api_url = self.config.rest_url() + "cloudaccounts"


    def get_cloudAccounts(self, sessionID, verify_ssl=False):
        #params = {'sID': sessionID}
        cookies = dict(sID=sessionID)
        response = requests.get(self.api_url, verify=verify_ssl, headers=self.headers,cookies=cookies)
        return response.json()


    def get_cloudAccount(self, id, sessionID, verify_ssl=False):
        params = {'sID': sessionID}
        url = self.api_url + "/{}".format(id)
        print("URL: %s" % url)
        response = requests.get(url, verify=verify_ssl, headers=self.headers, params=params)
        return response.json()


    def test_connection(self, id:str, sessionID:str, verify_ssl:bool = False) -> Dict[str, str]:
        params = {'sID': sessionID }
        url = self.api_url + "/" + id + "/testconnection"
        print(url)
        response = requests.put(url, verify=verify_ssl, headers=self.headers, params=params)
        print(response) #TODO on error not connected dsm api returns xml when looking for JSON
        return response.json()


    def syncronize_account(self, id:str, sessionID:str, verify_ssl:bool = False) -> Dict[str, str]:
        params = {'sID': sessionID }
        url = self.api_url + "/" + id + "/synchronize"
        response = requests.put(url, verify=verify_ssl, headers=self.headers, params=params)
        return response.json()

    def add_cloudaccount_aws(self, access_key, secret_key, sessionID, verify_ssl=False):
        url = self.api_url + '/aws'
        headers = {'Content-Type': 'application/json'}
        cookies = dict(sID=sessionID)
        data=AddAwsRequest(access_key=access_key, secret_key=secret_key).to_json()
        r = requests.post(url, data=data, verify=verify_ssl, cookies=cookies, headers=headers)
        return json.dumps(dict(status_code=r.status_code))


    def add_cloudaccount_aws_cross_account(self, external_id, role_arn, sessionID, verify_ssl=False):
        url = self.api_url + '/aws'
        headers = {'Content-Type': 'application/json'}
        cookies = dict(sID=sessionID)
        data = AddAwsRequest(external_id=external_id, role_arn=role_arn).to_json()
        r = requests.post(url, data=data, verify=verify_ssl, cookies=cookies, headers=headers)
        return json.dumps(dict(status_code=r.status_code))



class AwsCredentials:

    def __init__(self, access_key, secret_key):
        self.access_key = access_key
        self.secret_key = secret_key

    def to_json(self):
        return dict(accessKeyId=self.access_key, secretKey=self.secret_key)

class CrossAccountRole:

    def __init__(self, external_id, role_arn):
        self.external_id = external_id
        self.role_arn = role_arn


    def to_json(self):
        return dict(externalId=self.external_id, roleArn=self.role_arn)


class AddAwsRequest:

    def __init__(self, access_key=None, secret_key=None, external_id=None, role_arn=None):
        self.access_key = access_key
        self.secret_key = secret_key
        self.external_id = external_id
        self.role_arn = role_arn


    def to_json(self):
        if self.access_key is None:
            return json.dumps(dict(AddAwsAccountRequest=dict(crossAccountRole=CrossAccountRole(self.external_id, self.role_arn).to_json())))
        else:
            return json.dumps(dict(AddAwsAccountRequest=dict(awsCredentials=AwsCredentials(self.access_key, self.secret_key).to_json())))






from datetime import datetime
from typing import Dict
import requests


from ..config import Config



class UsageUtils:


    def __init__(self, config):
        self.config = config
        self.api_url = self.config.rest_url() + "monitoring/usages"
        self.headers = {'Accept': 'application/json'}

    def jvm_usage(self, sessionID:str, manager_node_id: str, from_date: datetime, to_date: datetime, verify_ssl:bool = False) -> Dict[str, str]:
        params = {'sID': sessionID }

        if len(manager_node_id) > 0: params['managerNodeID'] = manager_node_id
        if from_date is not None: params['from']= from_date.ctime()
        if to_date is not None: params['to'] = to_date.ctime()

        url = self.api_url + "/jvm"
        print(params)
        response = requests.get(url, verify=verify_ssl, headers=self.headers, params=params)
        return response.json()